﻿define("gl-translations/asset/view-model/HierarchicalListViewModel", [
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/topic",
    "dojo/aspect",

    "dojo/Stateful",
    "dojo/when",
    "dijit/registry",
    "dijit/Destroyable",
    //epi
    "epi",
    "epi/dependency",

    "epi/shell/ClipboardManager",
    "epi/shell/command/_CommandProviderMixin",
    "epi/shell/selection",
    "epi/shell/TypeDescriptorManager",
    //epi-cms
    "epi-cms/_MultilingualMixin",
    "epi-cms/ApplicationSettings",
    "epi-cms/contentediting/_ContextualContentContextMixin",
    "epi-cms/core/ContentReference",
    "epi-cms/widget/ContentForestStoreModel",
    "epi-cms/widget/ContextualContentForestStoreModel",
    //command
    "epi-cms/widget/CreateCommandsMixin",
    "gl-translations/component/command/Translate",
    "gl-translations/component/command/AddToJob",
    "gl-translations/component/command/AddToNewJob",
    "epi-cms/command/CutContent",
    "gl-translations/component/command/Delete",
    "epi-cms/widget/ContentTreeStoreModel",
    "epi-cms/widget/ContentTreeModelConfirmation",
    "epi-cms/asset/command/ChangeContextToSelection",
    "epi-cms/plugin-area/assets-pane"
],

function (
// dojo
    array,
    declare,
    lang,
    topic,
    aspect,

    Stateful,
    when,
    registry,
    Destroyable,
    // epi
    epi,
    dependency,

    ClipboardManager,
    _CommandProviderMixin,
    Selection,
    TypeDescriptorManager,
    //epi-cms
    _MultilingualMixin,
    ApplicationSettings,
    _ContextualContentContextMixin,
    ContentReference,
    ContentForestStoreModel,
    ContextualContentForestStoreModel,
    //command
    CreateCommandsMixin,
    TranslateContentCommand,
    AddToJobCommand,
    AddToNewJobCommand,
    CutContentCommand,
    DeleteContentCommand,
    ContentTreeStoreModel,
    ContentTreeModelConfirmation,
    ChangeContextToSelectionCommand,
    assetsPanePluginArea
) {

	return declare([Stateful, _ContextualContentContextMixin, _MultilingualMixin, CreateCommandsMixin, Destroyable], {
		// summary:
		//      Handles search and tree to list browsing widgets.
		//
		// tags:
		//      internal

		// clipboardManager: [const] ClipboardManager
		//      Used to handle copy-paste operations with the commands.
		clipboardManager: null,

		// selection: [const] Selection
		//      Used to handle currently selected items for the commands.
		selection: null,

		// commands: [readonly] _Command[]
		//      Used to handle currently selected items for the commands.
		commands: null,

		// createCommands: [readonly] _Command[]
		//      Used to handle currently selected items for the commands.
		//createCommands: null,

		// createHierarchyCommands: [readonly] _Command[]
		//      Used to handle currently selected items for the commands.
		createHierarchyCommands: null,

		// pseudoContextualCommands: [readonly] _Command[]
		//      Used to handle currently selected items for the commands.
		pseudoContextualCommands: null,

		// treePaths: [public] Array[]
		//      An array of arrays containing a hierarchy of content items starting with the root
		//      and ending with the actual item being selected.
		treePaths: null,

		// selectedTreeItems: [public] Array
		//      The currently selected tree items.
		selectedTreeItems: null,

		// selectedListItems: [public] Array
		//      An array containing the currently selected list items.
		selectedListItems: null,

		// listQuery: Query
		//      Query object holding parameters to get the children.
		//      of the current tree item
		listQuery: null,

		// listQueryOptions: Object
		//      Extra parameters needed to query the store for the list items.
		listQueryOptions: null,

		// showAllLanguages: Boolean
		//      Indicates if to query for items only in current language context or not.
		showAllLanguages: true,

		// treeStoreModelClass: [const] Function
		//      Class to use as model for the tree.
		treeStoreModelClass: null,

		// treeStoreModel: [const] TreeStoreModel
		//      TreeStoreModel instance.
		treeStoreModel: null,

		// store: [const] Dojo/Store
		//      Store instance used for all server queries.
		store: null,

		// searchStore: [const] Dojo/Store
		//      Store instance used for all server search queries.
		searchStore: null,

		// storeKey: [const] String
		//      Key to resolve store from dependency.
		storeKey: "epi.cms.content.light",

		// mainNavigationTypes: String[]
		//      Which types to filter for tree queries. Also used with trash command.
		mainNavigationTypes: null,

		// containedTypes: [const] String[]
		//       Which types to filter for list queries.
		containedTypes: null,

		// noDataMessages: [public] Object
		//      Object containing the texts for when the list contains no items. It should have two properties named single and multiple.
		noDataMessages: null,

		// noDataMessage: [public] String
		//      String to be displayed in the list when no items are returned from the store.
		noDataMessage: "",

		// profile: [private] shell/Profile
		//       The current users profile that is used to store settings.
		_profile: null,

		// _commandItems: [private] Array
		//      An array containing the currently selected items used in the commands.
		_commandItems: null,

		// _pluginCommands: [private] Array
		//      An array containing the commands loaded from the plugin area.
		//_pluginCommands: null,

		constructor: function () {

			// Create the command registry
			this._commandRegistry = {
				sort: function () {
					var commands = [];
					for (var key in this) {
						if (key !== "toArray" && key !== "sort" && this.hasOwnProperty(key)) {
							var index = this[key].order;
							if (!index) {
								index = 100;
							}
							commands.push([index, this[key].command]);
						}
					}

					commands.sort(function (a, b) {
						return a[0] - b[0];
					});

					return commands;
				},
				toArray: function () {
					var sortedCommand = this.sort();
					var commands = [];
					array.forEach(sortedCommand, function (key) {
						commands.push(key[1]);
					});

					return commands;
				}
			};

			this.own(assetsPanePluginArea.on("added, removed", this._setCommands.bind(this)));
		},

		postscript: function (args) {
			this.inherited(arguments);
			this.contentRepositoryDescriptors = this.contentRepositoryDescriptors || dependency.resolve("epi.cms.contentRepositoryDescriptors");
			declare.safeMixin(this, this.contentRepositoryDescriptors.get(args.repositoryKey));

			this._profile = this._profile || dependency.resolve("epi.shell.Profile");
			this.clipboardManager = this.clipboardManager || new ClipboardManager();
			this.selection = this.selection || new Selection();
			this.store = this.store || dependency.resolve("epi.storeregistry").get(this.storeKey);
			this.searchStore = this.searchStore || dependency.resolve("epi.storeregistry").get("epi.cms.content.search");

			this._setupTreeStoreModel();

			this._setupCommands();
			this._setCommands();
			this.set("listQueryOptions", this.treeStoreModel._queryOptions);
		},

		startup: function () {
			// summary:
			//      Allows the view model to start reacting to external input.
			// tags:
			//      protected

			this.inherited(arguments);

			this._setupSelection();
		},

		getCommand: function (commandName) {
			// summary:
			//      Gets a command by command name
			// tags:
			//      protected

			return this._commandRegistry[commandName] ? this._commandRegistry[commandName].command : null;
		},

		

		updateTreePaths: function (rootChildItems) {
			// summary:
			//      Updates the treePaths value by comparing the existing treePaths with the new child items of the tree root.
			// rootChildItems:
			//      The new child items to the tree's root.
			// tags:
			//      public

			var treePaths = this.get("treePaths");

			if (!treePaths || !treePaths.length) {
				return;
			}
		},

		_isSupportedContent: function (/*Object*/content) {
			// summary:
			//      Indicates whether the given content is a type contained by this widget.
			// content:
			//      Object to validate
			// tags:
			//      private
			return !!(content && content.id) && this.containedTypes.some(function (type) {
				return TypeDescriptorManager.isBaseTypeIdentifier(content.dataType, type);
			});
		},

		setTreePaths: function (treePaths) {
			// summary:
			//      This is called whenever a user clicks in the components folder tree.
			// treePaths: Array[]
			//      An array of arrays containing a hierarchy of content items starting with the root
			//      and ending with the actual item being selected.
			// tags:
			//      public

			// We call the custom setter directly here instead of using this.set("treePaths") because we
			// don't want want to update any watchers. The tree in the component should already have the correct selection.
			if (!epi.areEqual(treePaths, this.get("treePaths"))) {
				this._treePathsSetter(treePaths);
			}
		},

		_treePathsSetter: function (value) {
			// summary:
			//      Stores the treePaths value and resets the list selection.
			//      Calling set("treePaths", value) should update the corresponding components tree selection.
			// value: Array[]
			//      An array of arrays containing a hierarchy of content items starting with the root
			//      and ending with the actual item being selected.
			// tags:
			//      protected

			this.treePaths = value;


			// By using this._changeAttrValue instead of this.set("selectedListItems", null) we will update any
			// watchers (IE update selection in list component) without forcing this model to update the commands
			// several times in a row.
			this._changeAttrValue("selectedListItems", null);

			var treePaths = lang.clone(value);
			// The last item in every treePath array is the item that is selected.
			// We grab the last item and store that as the selectedTreeItems property.
			var selectedItems = treePaths ? treePaths.map(function (path) {
				return path.pop();
			}) : null;

			this.set("selectedTreeItems", selectedItems);
		},

		_selectedTreeItemsSetter: function (value) {
			// summary:
			//      Updates the commands with the specified value and updates the list query.
			// value:
			//      The selected tree items.
			// tags:
			//      protected

			this.selectedTreeItems = value;
			this._updateListQuery(value);

			if (!this.get("isSearching")) {
				this._updateCommands(value);
			}
		},

		setSelectedListItems: function (selectedListItems) {
			// summary:
			//      This is called whenever a user clicks in the components content list.
			// selectedListItems:
			//      The items that is currently selected in the list.
			// tags:
			//      public

			// We call the custom setter directly here instead of using this.set("selectedListItems") because we
			// don't want want to update any watchers. The list in the component should already have the correct selection.
			this._selectedListItemsSetter(selectedListItems);
		},

		_selectedListItemsSetter: function (value) {
			// summary:
			//      Updates the commands with the specified value.
			//      Calling set("selectedTreeItems", value) should update the corresponding components list selection.
			// value:
			//      The selected list items.
			// tags:
			//      protected

			this.selectedListItems = value;

			if (!this.get("isSearching")) {
				this._updateCommands(value);
			}
		},

		setSelectedSearchResults: function (selectedSearchResults) {
			// summary:
			//      This is called whenever a user clicks in the components search results.
			// selectedSearchResults:
			//      The items that is currently selected in the search results.
			// tags:
			//      public

			// We call the custom setter directly here instead of using this.set("selectedSearchResults") because we
			// don't want want to update any watchers. The search results in the component should already have the correct selection.
			this._selectedSearchResultsSetter(selectedSearchResults);
		},

		_selectedSearchResultsSetter: function (value) {
			// summary:
			//      Updates the commands with the specified value.
			// value:
			//      The selected search results.
			// tags:
			//      protected

			this.selectedSearchResults = value;
			this._updateCommands(value);
		},

		updateTreeCommands: function () {
			// summary:
			//      Updates the commands with the currently selected tree items.
			// tags:
			//      public

			this._updateCommands(this.get("selectedTreeItems"));
		},

		updateListCommands: function () {
			// summary:
			//      Updates the commands with the currently selected list items.
			// tags:
			//      public
			this._updateCommands(this.get("selectedListItems"));
		},

		_setupSelection: function () {
			// summary:
			//      Get target tree item and list item for selection
			// tags:
			//      protected

			when(this.getCurrentContext(), function (ctx) {
                var paths = this._getDefaultTreePaths();
                when(paths).then(function (paths) {
					this.set("noDataMessage", "");
					 if(Array.isArray(paths[0])) {
						var last = paths[0][paths[0].length - 1]
						if(last.statusCode && last.statusCode == 403){
							this.set("noDataMessage", "You do not have access rights to view jobs");
						}
					 }
                    this.set("treePaths", paths);
                }.bind(this));
			}.bind(this));
		},

		_getDefaultTreePaths: function () {
			// summary:
			//      Gets a tree paths that contains the treeStoremodel root and the first root item.
			// tags:
			//      private

			// TODO: Fix this hack of always using the first item in roots
			var rootId = ContentReference.toContentReference(this.roots[0]).toString();
			return this.store.get(rootId).then(function (model) {
				return [[this.treeStoreModel.root, model]];
			}.bind(this));
		},

		_setupTreeStoreModel: function () {
			// summary:
			//      Creates an configures the treeStoreModel.
			// tags:
			//      protected

			if (!this.treeStoreModelClass) {
				//Assign the default tree store model class if no custom one has been assigned.
				this.treeStoreModelClass = this.enableContextualContent ? ContextualContentForestStoreModel : ContentForestStoreModel;
			}

			var treeModel = this.treeStoreModel || new this.treeStoreModelClass({
				store: this.store,
				roots: this.roots,
				typeIdentifiers: this.mainNavigationTypes,
				containedTypes: this.containedTypes,
				notAllowToCopy: this.preventCopyingFor,
				notAllowToDelete: this.preventDeletionFor,
				notSupportContextualContents: this.preventContextualContentFor,
				autoSelectPastedItem: false,
				onAddDelegate: lang.hitch(this, function (node) {
					var targetNode = registry.getEnclosingWidget(node.domNode),
                        target = targetNode && targetNode.item,
                        canExecute = (typeof (this.treeStoreModel.canEdit) === "function") && this.treeStoreModel.canEdit(target);

					if (canExecute) {
						node.edit();


						// Publish the children change AFTER the user has change the name of the new folder or canceled the editor
						// REMARK: We need to do this after the change otherwise the editing will be canceled because the children change replaced the child items in the tree
						// TODO: This should be handled by the tree model
						var publish = function () {
							topic.publish("/epi/cms/contentdata/childrenchanged", target.parentLink);
						};

						var handle = node.on("rename", function () {
							handle.remove();

							publish();
						});

						var cancelHandle = node.on("cancelEdit", function () {
							cancelHandle.remove();

							publish();
						});

					}
				}),
				onPasteItemsFailed: lang.hitch(this, this._pasteItemsFailed),
				additionalQueryOptions: {
					sort: this._getSortSettings()
				}
			});

			// The ContentTreeModel is a destroyable, so we need to own it and clean up when this model is destroyed
			this.own(treeModel, aspect.after(treeModel, "rename", this._afterRename.bind(this)));

			this.set("treeStoreModel", treeModel);
		},

		_afterRename: function (renamePromise) {
			// summary:
			//      Updates selectedTreeItems property when treeModel rename is executed.
			// tags:
			//      private

			return renamePromise.then(function (renamedItem) {
				// the item returned by rename method came from ContentDataStore
				// and selectedTreeItems expects data from ContentStore
				return this.store.get(renamedItem.contentLink)
                    .then(function (item) {
                    	this.set("selectedTreeItems", [item]);
                    }.bind(this));
			}.bind(this));
		},

		_getSortSettings: function () {
			// summary:
			//      Returns the list of sort criteria.
			// tags:
			//      protected

			return [{ attribute: "name", descending: false }];
		},

		_setupCommands: function () {
			// summary:
			//      Creates and registers the commands used.
			// tags:
			//      protected

			var settings = {
				category: "context",
				model: this.treeStoreModel,
				selection: this.selection,
				clipboard: this.clipboardManager,
				context: this._currentContext
            };

            var settings2 = {
                category: "context",
                model: new ContentTreeModelConfirmation(new ContentTreeStoreModel()),
                selection: this.selection,
                clipboard: this.clipboardManager,
context: this._currentContext
            };

			this.createHierarchyCommands = {};
			var index = 1;

			this.createCommands = this.getCreateCommands(index);

			var editableTypes = array.filter(this.containedTypes, function (item) {
				return this.mainNavigationTypes.indexOf(item) < 0;
			}, this);

			var commands = {
				translate: {
					command: new TranslateContentCommand(settings),
					order: 1
				},
				edit: {
					command: new ChangeContextToSelectionCommand(lang.mixin({
						forceReload: true,
						forceContextChange: true,
						typeIdentifiers: editableTypes
					}, settings)),
					order: 3
                },
                "add": {
                    command: new AddToJobCommand(settings),
                    order: 10
                    
                },
                "create": {
                    command: new AddToNewJobCommand(settings),
                    order: 15
                    
                },
				"delete": {
                    command: new DeleteContentCommand(settings2),
					order: 50
				},
			};

			this._commandRegistry = lang.mixin(this._commandRegistry, this.createHierarchyCommands, commands);
			this.pseudoContextualCommands = this._getPseudoContextualCommands();
		},

		_setCommands: function () {
			// summary:
			//      Sets the commands from the hierarchical list plugin area and the command registry
			// tags:
			//      private

			var commands = this._commandRegistry.toArray();//.concat(pluginCommands);
			this.set("commands", commands);
		},

		_pasteItemsFailed: function (ignoredItems) {
			// summary:
			//      Configures the items list based on the result of the paste operation.
			// tags:
			//      private

			this.set("selectedListItems", ignoredItems);
		},

		_updateListQuery: function (items) {
			// summary:
			//      Creates a new query and updates the listQuery property.
			// tags:
			//      protected

			var query = null;

			if (items && items.length > 0) {
				query = this._createListChildrenQuery(items);
				//this.set("noDataMessage", items.length > 1 ? this.noDataMessages.multiple : this.noDataMessages.single);
			}

			this.set("listQuery", query);
		},

		_createListChildrenQuery: function (items) {
			// summary:
			//      Creates a new query .
			// tags:
			//      private

			// remove all mainNavigationTypes from containedTypes, to avoid of displaying Folder in Content List
			var contentTypes = array.filter(this.containedTypes, function (item) {
				return this.mainNavigationTypes.indexOf(item) < 0;
			}, this);

			var ids = items.map(function (item) {
				return item.contentLink.toString();
			});

			return { references: ids, query: "getchildren", allLanguages: this.showAllLanguages, typeIdentifiers: contentTypes };
		},

		_updateCommands: function (selectedItems) {
			// summary:
			//      Updates the current model for all commands needing this.
			// tags:
			//      protected
			
			this._commandItems = selectedItems;

			this._updateSelection(selectedItems);

			if (selectedItems && selectedItems.length === 1) {
				this.decoratePseudoContextualCommands(this.pseudoContextualCommands);
			}
		},

		_updateSelection: function (selectedItems) {
			// summary:
			//      Updates the selection manager to the selected items. Used when commands execute.
			// tags:
			//      protected

			var selectionData = selectedItems ? selectedItems.map(function (selectedItem) {
				return { type: "epi.cms.contentdata", data: selectedItem };
			}) : [];
			this.selection.set("data", selectionData);
		},


		_getPseudoContextualCommands: function () {
			// summary:
			//      Get commands to decorates
			// returns: [Array]
			//      Array of command object that each is instance of "epi.shell.command._Command" class
			// tags:
			//      private

			var key,
                commands = [];

			for (key in this.createHierarchyCommands) {
				commands.push(this.createHierarchyCommands[key].command);
			}
			return commands;
		},
	});
});
