﻿define("gl-translations/component/command/AddToJob", [
    "dojox/layout/ResizeHandle",
    "dojo/on",
    "dojo/_base/connect",
    "dojo/parser",
    "dojo/_base/lang",
    "epi/routes",
    "dojo/when",
    "dojo/topic",
    "epi/string",
    "dojo/_base/declare",
    "epi/shell/widget/dialog/Alert",
    "epi/dependency",
    "epi/shell/command/_Command",
    "epi/shell/TypeDescriptorManager",
    "epi-cms/legacy/LegacyDialogPopup",
    "epi-cms/widget/_HasChildDialogMixin",
    "epi-cms/core/ContentReference",
    "epi/shell/ViewSettings",
    "epi/shell/socket/hub",
    "epi/shell/command/_SelectionCommandMixin",
    "epi-cms/_ContentContextMixin",
    "epi-cms/contentediting/ContentActionSupport"
],

    function (
        ResizeHandle, on,
        connect,
        parser,
        lang,
        routes,
        when,
        topic,
        epistring,
        declare,
        Alert,
        dependency,
        _Command,
        TypeDescriptorManager,
        LegacyDialogPopup,
        _HasChildDialogMixin,
        ContentReference,
        ViewSettings,
        hub,
        _SelectionCommandMixin,
        _ContentContextMixin,
        ContentActionSupport
    ) {

        return declare([_Command, _HasChildDialogMixin, _SelectionCommandMixin, _ContentContextMixin], {

            label: "Add current item to job",

            iconClass: "epi-iconShare",
            projectJobId: null,
            currentContext: null,
            dataObject: null,
            constructor: function () {

                // summary:
                //        Constructs the object and sets up a reference to the content data store.
                // tags:
                //        public


            },

            _onItemRemoved: function (dataObject) {
                // Move item from gadget to trash in UI
                var contentWithoutVersion = (new ContentReference(this.model._currentContext.id)).createVersionUnspecificReference().toString();
                var dataObjectWithoutVersion = (new ContentReference(dataObject)).createVersionUnspecificReference().toString();
                if (dataObjectWithoutVersion === contentWithoutVersion) {
                    topic.publish("/epi/shell/context/request", { uri: ViewSettings.settings.defaultContext });
                }
                this.model.contentDataStore.evict(dataObjectWithoutVersion);
                this.model.contentDataStore.refresh(dataObjectWithoutVersion);
            },

            setLabel: function (newLabel) {
                this.label = newLabel;
            },
            _execute: function () {

                if (this.selection) {
                    this.projectJobId = this.selection.data[0].data.contentLink;
                    this.dataObject = this.selection.data[0].data;
this.currentContext = this._currentContext.id;
                }
                else {
                    this.projectJobId = this.model.contentLink;
                    this.dataObject = this.model.contentLink;
this.currentContext = null;
                }

                this.store = this.store || dependency.resolve("epi.storeregistry").get("globallink.translationprojectstore");
                var contentWithoutVersion = (new ContentReference(this.projectJobId)).createVersionUnspecificReference().toString();
                var contentWithWorkingId = (new ContentReference(this.projectJobId)).toString();
                var promise = this.store.put({
                    "projectId": contentWithWorkingId,
                    "contentId": this.currentContext,
                    "action": "addToJob"
                });
                var self = this;
                when(promise, function (value) {
                    console.log(value);
                    var dialog = new Alert({
                        heading: "Item added to job",
                        destroyOnHide: true,
                        description: "Item added to job"
                    });
                    dialog.show();
                }, function (error) {
                    var dialog = new Alert({
                        heading: "Cannot add to job",
                        destroyOnHide: true,
                        description: epistring.toHTML(error.xhr.statusText)
                    });
                    dialog.show();
                });
            },
            

            destroy: function () {
                if (this._dialog) {
                    this._dialog.destroyRecursive();
                    delete this._dialog;
                }
                this.inherited(arguments);
            },

            _onDialogHide: function (dataObject) {
                if (this._dialog) {
                    this._dialog.hide();
                }


                var contentUri = this.currentContext.getCurrentContext().versionAgnosticUri;
                topic.publish("/epi/shell/context/request", { uri: contentUri });
            },

            _getLegacyDialogParameters: function () {
                var width = $('html', window.parent.document).width() * 88 / 100;
                var height = $('html', window.parent.document).height() * 70 / 100;
                return {
                    autoFit: true,
                    features: {
                        width: width,
                        height: height
                    }
                };
            },

            _getLegacyDialogUrl: function () {

                var params = {};
                if (this.projectJobId) {
                    params.jobId = dojo.toJson(this.projectJobId);
                }

                return routes.getActionPath(params, this.translationDialogUrl);
            },


            _onModelChange: function () {

                var model = this.model,
                    selection = this.selection.data,
                    canAction = function (item) {

                        var createAction = ContentActionSupport.action.Create;
                        var canCreate = ContentActionSupport.hasAccessToAction(item.data, createAction, null, true);

                        var editAction = ContentActionSupport.action.Edit;
                        var canEdit = ContentActionSupport.hasAccessToAction(item.data, editAction, null, true);

                        var adminAction = ContentActionSupport.action.Administer;
                        var canAdmin = ContentActionSupport.hasAccessToAction(item.data, adminAction, null, true);

                        return canCreate && canEdit && canAdmin;
                    };

                var target = this._getSingleSelectionData();
                if (!selection) {
                    this.set("canExecute", false);
                    this.set("isAvailable", false);
                    return;
                }

                var canExecute = true;
                if (selection.length == 1) {
                    canExecute = this._isTargetOfCorrectType(selection[0].data);
                    var validTypeIdentifier = "globallink.translation.core.translationproject";
                    if (selection[0].data.typeIdentifier != validTypeIdentifier) {
                        this.set("isAvailable", false);
                        canExecute = false;
                    } else {
                        this.set("isAvailable", true);
                    }
                    this.setLabel( "Add current item to " + selection[0].data.name);
                } else {
                    this.set("isAvailable", false);
                }
                this.set("isAvailable", true);
                this.set("canExecute", !!(model && selection.length) && selection.every(canAction) && this._currentContext && canExecute);
            },

            _isTargetOfCorrectType: function (target) {
                // summary:
                //      If this.typeIdentifiers is defined we will check if the model is of correct type,
                //      otherwise we will always return true.
                // tags:
                //      private

                if (!this.typeIdentifiers) {
                    return true;
                }
                return this.typeIdentifiers.some(function (type) {
                    return TypeDescriptorManager.isBaseTypeIdentifier(target.typeIdentifier, type);
                }, this);
            },


            _updateAvailable: function () {

                var validTypeIdentifier = "globallink.translation.core.translationproject";
                var isAvailable = TypeDescriptorManager.isBaseTypeIdentifier(this.model ? (this.model.contentData ? this.model.contentData.typeIdentifier : "") : "", validTypeIdentifier);
                var containedTypes = this.model.containedTypes;
                if (this.model && containedTypes && containedTypes.includes(validTypeIdentifier) && this._currentContext) {
                    isAvailable = true;
                }

                this.set("isAvailable", !!isAvailable);
            }
        });
    });
