﻿define("gl-translations/component/command/SubmitForTranslation", [
    "dojox/layout/ResizeHandle",
    "dojo/on",
    "dojo/_base/connect",
    "dojo/parser",
    "dojo/_base/lang",
    "epi/routes",
    "dojo/when",
    "dojo/topic",
    "epi/string",
    "dojo/_base/declare",
    "epi/shell/widget/dialog/Alert",
    "epi/shell/DialogService",
    "epi/dependency",
    "epi/shell/command/_Command",
    "epi/shell/TypeDescriptorManager",
    "epi-cms/legacy/LegacyDialogPopup",
    "epi-cms/widget/_HasChildDialogMixin",
    "epi-cms/core/ContentReference",
    "epi/shell/ViewSettings",
    "epi/shell/socket/hub",
    "epi/shell/command/_SelectionCommandMixin",
    "epi-cms/_ContentContextMixin",
    "epi-cms/contentediting/ContentActionSupport",
    "dijit/layout/_LayoutWidget",
    "epi/shell/widget/_ActionProviderWidget"
],

    function (
        ResizeHandle, on,
        connect,
        parser,
        lang,
        routes,
        when,
        topic,
        epistring,
        declare,
        Alert,
        DialogService,
        dependency,
        _Command,
        TypeDescriptorManager,
        LegacyDialogPopup,
        _HasChildDialogMixin,
        ContentReference,
        ViewSettings,
        hub,
        _SelectionCommandMixin,
        _ContentContextMixin,
        ContentActionSupport,
        _LayoutWidget,
        _ActionProviderWidget
    ) {

        return declare([_Command, _HasChildDialogMixin, _ContentContextMixin], {

            label: "Translate",

            iconClass: "epi-iconTranslate",
            translationDialogUrl: "ClientResources/Scripts/Asset/Templates/TranslationConfigurator.aspx",
            projectJobId: null,
            dataObject: null,
            constructor: function () {

                // summary:
                //        Constructs the object and sets up a reference to the content data store.
                // tags:
                //        public

            },

            setLabel: function (newLabel) {
                this.label = newLabel;
            },
            _execute: function () {

                if (this.selection) {
                    this.projectJobId = this.selection.data[0].data.contentLink;
                    this.dataObject = this.selection.data[0].data;
                }
                else {
                    this.projectJobId = this.model.contentLink;
                    this.dataObject = this.model.contentLink;
                }

                this.store = this.store || dependency.resolve("epi.storeregistry").get("globallink.translationprojectstore");
                var contentWithoutVersion = (new ContentReference(this.projectJobId)).createVersionUnspecificReference().toString();
                var contentWithWorkingId = (new ContentReference(this.projectJobId)).toString();
                var promise = this.store.get(contentWithWorkingId);
                var self = this;
                when(promise, function (value) {
                    self._showDialog();
                }, function (error) {
                    var dialog = new Alert({
                        heading: "Cannot submit for Translation",
                        destroyOnHide: true,
                        description: epistring.toHTML(error.xhr.statusText)
                    });
                    dialog.show();
                });
            },

            _showDialog: function () {
                if (!this._dialog || this._dialog._destroyed) {
                    var params = this._getLegacyDialogParameters();
                    var cmsPath = routes.getActionPath({
                        moduleArea: "CMS",
                        controller: "Home",
                        action: "Index"
                    });
                    var rootPath = cmsPath.split("/")[1];
                    var legacyDialogUrl = this._getLegacyDialogUrl();
                    this._dialog = new LegacyDialogPopup(lang.mixin(params, {
                        url: routes.getActionPath({
                            path: legacyDialogUrl + "?id=" + this.projectJobId,
                            moduleArea: "globallink.translation"
                        }, "/" + rootPath + "/{moduleArea}/{path}"),
                        dialogArguments: window.document,
                        onHide: this._onDialogHide,
                        currentContext: this
                    }));

                    this._dialog.on("Load", lang.hitch(this, function () {
                        var iframeWindow = arguments[0].containerIframe.contentWindow;
                        iframeWindow.addEventListener("submitTranslation", lang.hitch(this, function () {

                            this._onDialogHide();
                        }));

                        //Show pop-up confirmation message right after Translation Job Wizard is closed
                        iframeWindow.addEventListener("showConfirmDialogForTranslationJob", lang.hitch(this, function (data) {
                            var cmsPath = routes.getActionPath({
                                moduleArea: "CMS",
                                controller: "Home",
                                action: "Index"
                            });
                            var dashboardUrl = routes.getActionPath({
                                path: "Dashboard",
                                moduleArea: "globallink.translation"
                            }, "/" + cmsPath.split("/")[1] + "/{moduleArea}/{path}")
                            DialogService.confirmation({
                                confirmActionText: "View Dashboard",
                                cancelActionText: "Close",
                                heading: 'GlobalLink Translation Jobs',
                                destroyOnHide: true,
                                defaultActionsVisible: false,
                                description: data.detail.description
                            }).then(function () {
                                window.location = dashboardUrl;
                            }).otherwise(function () {
                                
                            });
                            
                        }));
                        iframeWindow.addEventListener("showConfirmMessageForTranslationJob", lang.hitch(this, function (data) {
                            var dialogConfirm = new Alert({
                                heading: 'GlobalLink Translation Jobs',
                                destroyOnHide: true,
                                description: data.detail.description
                            });
                            dialogConfirm.show();
                        }));
                    }));
                };
                let id = this._dialog.id;
                var height = $('html', window.parent.document).height();
                $('.dijitDialogPaneContentArea', window.parent.document).css('height', height.toString() + 'px');
                let width = jQuery('html', window.parent.document).width() * 88 / 100;
                if (width < 1500) {
                    $('#' + id, window.parent.document).css('height', '86%');
                    $('.epi-dialogPaneContent').css('height', '94%');
                } else {
                    $('#' + id, window.parent.document).css('height', '84%');
                    $('.epi-dialogPaneContent').css('height', '95%');
                }
                let handle = new ResizeHandle({
                    targetId: id,
                    resizeAxis: "x"
                }).placeAt(id);
                this._dialog.show();

                on(handle, "resize", function (e) {
                    let width = $('html', window.parent.document).width() * 88 / 100;
                    if (parseInt(e.target.style.width) < width) {
                        document.getElementById(id).style.width = e.target.style.width;
                    } else {
                        $('#' + id).css('width', width + 'px');
                    }
                    if (parseInt(e.target.style.width) < 600) {
                        $('#' + id).css('width', '600px');
                    }
                    $('#' + id).find('iframe').parent().css('width', '100%');
                    $('#' + id).find('iframe').css('width', '100%');
                    // document.getElementById("uniqName_80_" + idWidth[idWidth.length - 1]).style.width = '100%';
                    // document.getElementById("uniqName_80_" + idWidth[idWidth.length - 1]).childNodes[1].style.width = '100%';
                });
            },

            destroy: function () {
                if (this._dialog) {
                    this._dialog.destroyRecursive();
                    delete this._dialog;
                }
                this.inherited(arguments);
            },

            _onDialogHide: function (dataObject) {
                if (this._dialog) {
                    this._dialog.hide();
                }


                //var contentUri = this.currentContext.getCurrentContext().versionAgnosticUri;
                //topic.publish("/epi/shell/context/request", { uri: contentUri });
            },

            _getLegacyDialogParameters: function () {
                var width = $('html', window.parent.document).width() * 88 / 100;
                var height = $('html', window.parent.document).height() * 70 / 100;
                return {
                    autoFit: true,
                    features: {
                        width: width,
                        height: height
                    }
                };
            },

            _getLegacyDialogUrl: function () {

                var params = {};
                if (this.projectJobId) {
                    params.jobId = dojo.toJson(this.projectJobId);
                }

                return routes.getActionPath(params, this.translationDialogUrl);
            },

            _onModelChange: function () {

                var model = this.model.contentData;
                var createAction = ContentActionSupport.action.Create;
                var canCreate = ContentActionSupport.hasAccessToAction(model, createAction, null, true);

                var editAction = ContentActionSupport.action.Edit;
                var canEdit = ContentActionSupport.hasAccessToAction(model, editAction, null, true);

                var adminAction = ContentActionSupport.action.Administer;
                var canAdmin = ContentActionSupport.hasAccessToAction(model, adminAction, null, true);

                var canExecute = canCreate && canEdit && canAdmin;
                this._updateAvailable();
                this.set("canExecute", canExecute);
            },

            _updateAvailable: function () {

                var validTypeIdentifier = "globallink.translation.core.translationproject";
                var isAvailable = TypeDescriptorManager.isBaseTypeIdentifier(this.model ? (this.model.contentData ? this.model.contentData.typeIdentifier : "") : "", validTypeIdentifier);
                var containedTypes = this.model.containedTypes;
                if (this.model && containedTypes && containedTypes.includes(validTypeIdentifier)) {
                    isAvailable = true;
                }

                this.set("isAvailable", !!isAvailable);
            }
        });
    });
