﻿define("gl-translations/component/command/TranslateProject", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/topic",
    "dojo/on",
    "dojo/when",
    "epi/string",
    "epi/dependency",
    "epi/shell/widget/dialog/Alert",
    "epi-cms/project/command/RemoveProjectConfirmation",
    // Resources
    "epi/i18n!epi/nls/episerver.cms.components.project.command.removeproject",
    // Parent class
    "epi-cms/project/command/_ProjectCommand"
], function (
    declare,
    lang,
    topic,
    on,
    when,
    epistring,
    dependency,
    Alert,
    RemoveProjectConfirmation,
    // Resources
    res,
    // Parent class
    _ProjectCommand
) {

        return declare([_ProjectCommand], {

            // category: [readonly] String
            //      A category which provides a hint about how the command could be displayed.
            category: "context",

            // label: [public] String
            //      The action text of the command to be used in visual elements.
            label: "Translate Project",

            // iconClass: [public] String
            //      The icon class of the command to be used in visual elements.
            iconClass: "epi-iconTranslate",

            _execute: function () {
                // summary:
                //      Executes this command assuming canExecute has been checked.
                // tags:
                //      protected
                var project = this.model.getProject();
                this.store = this.store || dependency.resolve("epi.storeregistry").get("globallink.translationprojectstore");
                var promise = this.store.post({
                    "projectId": project.id,
                    "contentId": "0",
                    "action": "translateProject"
                });

                when(promise)
                    .then(function (data) {
                        topic.publish("/epi/shell/context/request", {
                            uri: "epi.cms.contentdata:///" + data.contentLink
                        }, {
                                sender: this,
                                viewName: this.view,
                                forceContextChange: true,
                                forceReload: true
                            });
                    })
                    .otherwise(function (error) {
                        var dialog = new Alert({
                            heading: "Cannot Create Translation Job",
                            destroyOnHide: true,
                            description: epistring.toHTML(error.xhr.statusText)
                        });
                        dialog.show();
                    });
            },

            _removeProject: function (keepScheduling) {
                // summary:
                //      Removes the project on the server and reactivates the project items if keepScheduling is set to false.
                // tags:
                //      private

                var model = this.model,
                    project = model.getProject(),
                    promise = !keepScheduling && model.reactivateProject(project.id);

                when(promise)
                    .then(function () {
                        return model.removeProject();
                    })
                    .otherwise(function (error) {
                        console.error("An error occured while removing the project.", error);
                    });
            },

            _onPropertyChanged: function () {
                var project = this.model.getProject();
                // The command shouldn't be executable if a project is not selected and the project is deleted
                this.set("canExecute", !!project && !project.isDeleted);
            }
        });
    });
