﻿define("gl-translations/component/CustomAttributesEditor", [

    // Dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/on",
    "dojox/html/entities",

    // Dijit
    "dijit/focus",
    "dijit/_TemplatedMixin",
    "dijit/_Widget",
    "dijit/form/CheckBox",
    "dijit/form/TextBox",
    "dijit/form/Select",
    "dojo/store/Memory",
    "dojo/data/ObjectStore",

    // EPi
    "epi/shell/widget/_ValueRequiredMixin"
], function (

    // Dojo
    array,
    declare,
    lang,
    domConstruct,
    on,
    entities,

    // Dijit
    focusUtil,
    _TemplatedMixin,
    _Widget,
    CheckBox,
    TextBox,
    ComboBox,
    Memory,
    ObjectStore,

    // EPi
    _ValueRequiredMixin

) {

        return declare([_Widget, _TemplatedMixin, _ValueRequiredMixin], {
            // tags:
            //      internal

            templateString: "<div class=\"dijit dijitReset dijitInline\"></div>",

            baseClass: "epi-checkBoxList",

            valueIsCsv: true,

            valueIsInclusive: true,

            value: null,

            constructor: function () {
                this._checkboxes = [];
                this._labelClickHandles = [];
            },

            buildRendering: function () {
                this.inherited(arguments);

                //array.forEach(this.selections, this._addCheckBoxForItem, this);
            },

            destroy: function () {
                var checkbox;
                while ((checkbox = this._checkboxes.pop())) {
                    checkbox.destroyRecursive();
                }

                var labelClickHandle;
                while ((labelClickHandle = this._labelClickHandles.pop())) {
                    labelClickHandle.remove();
                }

                this.inherited(arguments);
            },

            focus: function () {
                // summary:
                //		Focus the widget.

                try {
                    if (this._checkboxes.length > 0) {
                        focusUtil.focus(this._checkboxes[0].domNode);
                    }
                } catch (e) {
                    /*quiet*/
                }
            },

            onChange: function (/*===== newValue =====*/) {
                // summary:
                //		Callback when this widget's value is changed.
                // tags:
                //		callback
            },

            onBlur: function () {
                // summary:
                //		Callback when this widget loses focus.
                // tags:
                //		callback
            },

            onFocus: function () {
                // summary:
                //		Callback when this widget gains focus.
                // tags:
                //		callback
            },

            _calculateValue: function (type) {
                if (valueIsInclusive) {
                    var values = [];
                    var storeChecked = this.valueIsInclusive;
                    array.forEach(this._checkboxes, function (checkbox) {
                        if (type == "text") {
                            values.push({ "name": checkbox.name, "value": checkbox.value });
                        } else if (type == "combo") {
                            values.push({ "name": checkbox.name, "value": checkbox.value });
                        } else {
                            values.push({ "name": checkbox.name, "value": checkbox.checked });
                        }
                    });

                    this._set("value", JSON.stringify(values));
                }
            },

            _setValueAttr: function (value) {
                this._set("value", value);

                var values = [];
                if (value && value!=null) {
                    values = JSON.parse(value);
                    if (this._checkboxes && this._checkboxes.length > 0) {
                        valueIsInclusive = false;
                        array.forEach(this._checkboxes, function (checkbox) {
                            var chk = values.filter(function (item, pos) {
                                return checkbox.name == item.name;
                            });
                            if (chk && chk.length > 0) {
                                checkbox.value = chk[0].value;
                                checkbox.set("value", chk[0].value);
                                //checkbox.onChange(chk[0].value);
                            }
                        });
                        valueIsInclusive = true;
                    }
                }
               


            },

            _compareCheckboxValue: function (checkbox, value) {
                // 'value' can be a different type from 'checkbox.value' so we stringify them before comparing.

                var checkboxValue = checkbox.value;

                if (checkboxValue === undefined || checkboxValue === null) {
                    return false;
                }

                return checkboxValue.toString() === value.toString();
            },

            _onBlur: function () {
                this.inherited(arguments);
                this.onBlur();
            },

            _addCheckBoxForItem: function (item) {
                // Create a container for the label and checkbox.
                var container = domConstruct.create("div", { "class": "epi-checkboxContainer" }, this.domNode);

                // Add the checkbox to the container.
                var checkbox = new CheckBox({ value: item.value, name: item.text, required: item.mandatory });
                var label = domConstruct.create("label", { "for": checkbox.id }, container);
                label.textContent = item.text +" ";
                checkbox.placeAt(container);

                this.own(
                    on(checkbox, "change", lang.hitch(this, function () {
                        this._calculateValue("checkbox");
                        this.onChange(this.value);
                    }))
                );

                checkbox.set("readOnly", !!this.readOnly);

                this._checkboxes.push(checkbox);

                // Add a label for the checkbox
                this._labelClickHandles.push(on(label, "click", lang.hitch(this, function () {
                    focusUtil.focus(checkbox.domNode);
                })));
            },
            //epi-card-list
            _addTextBoxForItem: function (item) {
                // Create a container for the label and checkbox.
                var container = domConstruct.create("div", { "class": "dijitInputField dijitInputContainer epi-form-container__section__row", "style": "padding:0" }, this.domNode);

                // Add the checkbox to the container.
                var textbox = new TextBox({ value: item.value, name: item.text, required: item.mandatory });
                var label = domConstruct.create("label", { "for": textbox.id, "style": (item.mandatory ? "color:#F00" : "")}, container);
                label.textContent = item.text + " ";
                textbox.placeAt(container);

                this.own(
                    on(textbox, "change", lang.hitch(this, function () {
                        this._calculateValue("text");
                        this.onChange(this.value);
                    }))
                );

                textbox.set("readOnly", !!this.readOnly);

                this._checkboxes.push(textbox);

                // Add a label for the checkbox
                
                this._labelClickHandles.push(on(label, "click", lang.hitch(this, function () {
                    focusUtil.focus(textbox.domNode);
                })));
            },

            _addComboBoxForItem: function (item) {
                // Create a container for the label and checkbox.
                var container = domConstruct.create("div", { "class": "epi-card-list epi-form-container__section__row", "style":"padding:0" }, this.domNode);

               // var stateStore = new Memory(item.values);
                var data = item.values.split(",").map(function (obj) {
                    return { id:obj, label: obj };
                });
                if (item.mandatory) {
                } else {
                    data.unshift({ id: "-1", label: " " });
                }
                var myStore = new Memory({ data: data });
                var os = new ObjectStore({ objectStore: myStore });
                // Add the checkbox to the container.
                var combobox = new ComboBox({
                    name: item.name,
                    store: os,
                    required: item.mandatory,
                    emptyLabel: "Choose value",
                    style: "width:15em;"
                });
                var label = domConstruct.create("label", { "for": combobox.id, "style": (item.mandatory ? "color:#F00" : "") }, container);
                label.textContent = item.text+" ";
                combobox.placeAt(container);

                this.own(
                    on(combobox, "change", lang.hitch(this, function () {
                        this._calculateValue("combo");
                        this.onChange(this.value);
                    }))
                );

                combobox.set("readOnly", !!this.readOnly);

                this._checkboxes.push(combobox);

            },

            _addEmptyBoxForItem: function () {
                var label = domConstruct.create("label", {}, this.domNode);
                label.textContent = "No custom attributes in selected GlobalLink Project";
                
            },

            filter: null,
            allSelections: null,
            _setFilterAttr: function (filter) {
                this._set("filter", filter);
                this.set("selections", this.allSelections);
            },

            _setSelectionsAttr: function (selections) {
                this.allSelections = selections;
                this._set("selections", array.filter(selections, this.filter || function () {
                    return true;
                }, this));

                this._checkboxes = [];
                domConstruct.empty(this.domNode);
                if (!this.selections || this.selections.length == 0) {
                    this._addEmptyBoxForItem();
                } else {
                    array.forEach(this.selections, function (selection) {
                        if (selection.type == "TEXT") {
                            this._addTextBoxForItem(selection);
                        } else if (selection.type == "COMBO") {
                            this._addComboBoxForItem(selection);
                        } else if (selection.type == "CHECKBOX") {
                            this._addCheckBoxForItem(selection);
                        }
                    }, this);
                    this.set("value", this.value);
                }
            },
        });
    });
