"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var api_url_1 = require("./api.url");
var http_client_1 = require("../common/shared/http-client");
require("rxjs/add/operator/toPromise");
var TranslationService = /** @class */ (function () {
    function TranslationService(httpClient) {
        this.httpClient = httpClient;
        //Pages: 1; Block: -2; Referenced Pages: -3; Product: -4;
        this.rootNodeIds = ["-1", "-2", "-3", "-4"];
    }
    TranslationService.prototype.allKeysToUpperCase = function (obj) {
        var output = {};
        for (var i in obj) {
            var key = i.charAt(0).toUpperCase() + i.slice(1);
            if (Object.prototype.toString.apply(obj[i]) === '[object Object]') {
                output[key] = this.allKeysToUpperCase(obj[i]);
            }
            else {
                if (Array.isArray(obj[i])) {
                    var arr = [];
                    for (var j = 0; j < obj[i].length; j++) {
                        var item = this.allKeysToUpperCase(obj[i][j]);
                        arr.push(item);
                    }
                    output[key] = arr;
                }
                else {
                    output[key] = obj[i];
                }
            }
        }
        return output;
    };
    TranslationService.prototype.getTranslations = function (projId) {
        var _this = this;
        var url = api_url_1.API.TranslationProject + '/info/' + projId;
        return this.httpClient.get(url).toPromise().catch().then(function (res) {
            return _this.allKeysToUpperCase(res.json()).Result;
        });
    };
    TranslationService.prototype.checkTreeModel = function (projId) {
        var _this = this;
        var url = api_url_1.API.getModelTree + "/checktree/" + projId;
        return this.httpClient.get(url).toPromise().catch().then(function (res) {
            var result = _this.allKeysToUpperCase(res.json()).Result;
            return _this.buildRootNodes(result);
        });
    };
    TranslationService.prototype.getTreeModel = function (projId) {
        var _this = this;
        var url = api_url_1.API.getModelTree + "/loadtree/" + projId;
        return this.httpClient.get(url).toPromise().catch().then(function (res) {
            var result = _this.allKeysToUpperCase(res.json()).Result;
            return _this.buildRootNodes(result);
        });
    };
    TranslationService.prototype.buildRootNodes = function (result) {
        var _this = this;
        var treeNodes = [];
        var pageRoot = {
            id: "-1",
            name: 'Pages',
            type: 'page',
            checked: true,
            children: result.Pages.map(function (page) { return _this.buildPageTreeNode(page); })
        };
        var blocksRoot = {
            id: '-2',
            type: 'block',
            checked: true,
            name: 'Blocks',
            children: result.Blocks.map(function (block) { return _this.buildBlockTreeNode(block); })
        };
        var refPages = {
            id: '-3',
            type: 'page',
            checked: true,
            name: 'Referenced Pages',
            children: result.ReferencedPages.map(function (refPage) { return _this.buildPageTreeNode(refPage); })
        };
        var commerceNodes = {
            id: '-4',
            type: 'Product',
            checked: true,
            name: 'Commerce Catalog',
            children: result.Commerce.map(function (refPage) { return _this.buildPageTreeNode(refPage); })
        };
        var refItems = result.ReferencedPages;
        //Build content tree
        if (result.Pages.length > 0) {
            treeNodes.push(pageRoot);
        }
        if (result.Blocks.length > 0) {
            treeNodes.push(blocksRoot);
        }
        if (result.Commerce.length > 0) {
            treeNodes.push(commerceNodes);
        }
        //Build Referenced Content
        if (result.ReferencedBlocks !== undefined && result.ReferencedBlocks.length > 0) {
            refItems = refItems.concat(result.ReferencedBlocks);
        }
        return {
            nodeItems: treeNodes,
            status: result.Status,
            excludedItems: result.ExcludeItems,
            refItems: refItems.map(function (refPage) { return _this.buildPageTreeNode(refPage); }),
            Id: result.Id !== undefined ? result.Id : ""
        };
    };
    TranslationService.prototype.buildPageTreeNode = function (page) {
        var _this = this;
        return {
            id: page.Id,
            name: page.Name,
            checked: page.Selected,
            isExpired: page.IsExpired,
            isDisabled: page.Selectable ? false : true,
            statusDetail: page.SelectedStatus,
            isExpanded: true,
            expanded: true,
            type: page.ContentType != null ? page.ContentType.toLowerCase() : 'page',
            children: page.ReferencedBlocks.map(function (childBlock) { return _this.buildBlockTreeNode(childBlock); }).concat(page.ReferencedContents.map(function (childPage) { return _this.buildPageTreeNode(childPage); }))
        };
    };
    TranslationService.prototype.buildBlockTreeNode = function (block) {
        var _this = this;
        return {
            id: block.Id,
            name: block.Name,
            type: 'block',
            isExpired: block.IsExpired,
            isDisabled: block.Selectable ? false : true,
            statusDetail: block.SelectedStatus,
            checked: block.Selected,
            children: block.ReferencedBlocks ? block.ReferencedBlocks.map(function (childBlock) { return _this.buildBlockTreeNode(childBlock); }).concat(block.ReferencedContents.map(function (childPage) { return _this.buildPageTreeNode(childPage); })) : ""
        };
    };
    TranslationService.prototype.createSubmission = function (projId, exclItms) {
        var _this = this;
        var url = api_url_1.API.TranslationProject + "/createSubmission";
        var body = JSON.stringify({ id: projId, "excludeItems": exclItms });
        return this.httpClient.post(url, body).toPromise().catch().then(function (res) {
            return _this.allKeysToUpperCase(res.json()).Result;
        });
    };
    TranslationService.prototype.updateSetting = function (projId, setting) {
        var _this = this;
        var url = api_url_1.API.TranslationProject + "/update/" + projId;
        return this.httpClient.post(url, setting).toPromise().catch().then(function (res) {
            var result = _this.allKeysToUpperCase(res.json()).Result;
            return _this.buildRootNodes(result);
        });
    };
    TranslationService.prototype.updatePostAction = function (projId, pAction) {
        var _this = this;
        var url = api_url_1.API.TranslationProject + "/updatepostaction";
        return this.httpClient.post(url, { "id": projId, "postAction": pAction }).toPromise().catch().then(function (res) {
            return _this.allKeysToUpperCase(res.json()).Result;
        });
    };
    TranslationService.prototype.updateSettings = function (projId, projSettings) {
        var _this = this;
        var url = api_url_1.API.TranslationProject + "/updateSettings/" + projId;
        return this.httpClient.post(url, projSettings).toPromise().catch().then(function (res) {
            var result = _this.allKeysToUpperCase(res.json()).Result;
            return _this.buildRootNodes(result);
        });
    };
    TranslationService.prototype.updateLocale = function (projId, locales) {
        var _this = this;
        var url = api_url_1.API.TranslationProject + "/updateLocales/" + projId;
        return this.httpClient.post(url, locales).toPromise().catch().then(function (res) {
            var result = _this.allKeysToUpperCase(res.json()).Result;
            return _this.buildRootNodes(result);
        });
    };
    TranslationService = __decorate([
        core_1.Injectable(),
        __metadata("design:paramtypes", [http_client_1.HttpClient])
    ], TranslationService);
    return TranslationService;
}());
exports.TranslationService = TranslationService;
//# sourceMappingURL=translation.service.js.map