﻿define("gl-translations/component/LocaleBlockContainer", [
    "dojo/_base/declare",
    "dojo/_base/lang",

    "epi/shell/layout/SimpleContainer"
],
    function (
        declare,
        lang,

        SimpleContainer
    ) {

        return declare([SimpleContainer], {
            ticketDropdown: null,
            sourceDropdown: null,
            targetsDropdown: null,
            customAttributes: null,

            addChild: function (child) {

                this.inherited(arguments);

                if (child.name.indexOf("locale.projectTicket") >= 0) {
                    this.ticketDropdown = child;
                    this.own(this.ticketDropdown.on("change", lang.hitch(this, this._updateSourceDropdown)));
                    this.own(this.ticketDropdown.on("change", lang.hitch(this, this._updateTargetsDropdown)));
                    this.own(this.ticketDropdown.on("change", lang.hitch(this, this._updateAttributesDropdown)));
                } else if (child.name.indexOf("locale.sourceLocale") >= 0) {
                    this.sourceDropdown = child;
                    this.own(this.sourceDropdown.on("change", lang.hitch(this, this._updateTargetsDropdown)));
                    this._updateSourceDropdown();
                } else if (child.name.indexOf("locale.targetLocales") >= 0) {
                    this.targetsDropdown = child;
                    this._updateTargetsDropdown();
                }
                else if (child.name.indexOf("locale.customAttributes") >= 0) {
                    this.customAttributes = child;
                    this._updateAttributesDropdown();
                }
            },

            _updateSourceDropdown: function () {
                var ticketValue = this.ticketDropdown.value;
                if (!ticketValue) {
                    return;
                }
                var selectedTicket = undefined;
				for(var i=0; i<this.ticketDropdown.selections.length;i++){
	   				var obj = this.ticketDropdown.selections[i];
                    if(obj.value == ticketValue)selectedTicket = obj;
                }
                var avaiableSources = selectedTicket.languageDirections.map(function(obj){
                    return obj.sourceLanguage;
                });

                avaiableSources = avaiableSources.filter(function (item, pos) {
                    return avaiableSources.indexOf(item) == pos;
                });

                var selectedSource = this.sourceDropdown.value;
                if (selectedSource != null && selectedSource.length > 0) {
                    var current = undefined;
					for(var i=0; i<this.sourceDropdown.selections.length;i++){
	   					var obj = this.sourceDropdown.selections[i];
                        if (obj.value === selectedSource)
                            current = obj
                    }
                    if (current !== undefined) {
                        this.sourceDropdown.set("value", current.value);
                    } else {
                        if (avaiableSources.indexOf(current.globalLinkLocale) == -1) {
                            if (avaiableSources.length == 1) {
                                var current = undefined;
                                for (var i = 0; i < this.sourceDropdown.selections.length; i++) {
                                    var obj = this.sourceDropdown.selections[i];
                                    if (obj.globalLinkLocale === avaiableSources[0])
                                        current = obj
                                }
                                if (current !== undefined) {
                                    this.sourceDropdown.set("value", current.value);
                                } else {
                                    this.sourceDropdown.set("value", null);
                                }
                            } else {
                                this.sourceDropdown.set("value", null);
                            }
                        }
                    }
                }

                var self = this;
                this.sourceDropdown.set("filter", function (source) {
                    var currentSource = undefined;
					for(var i=0; i<self.sourceDropdown.selections.length;i++){
	   				var obj = self.sourceDropdown.selections[i];
					if(obj.value === source.value)currentSource= obj
					}
                    return avaiableSources.indexOf(currentSource.globalLinkLocale) != -1;
                });
            },

            _updateTargetsDropdown: function () {
                var ticketValue = this.ticketDropdown.value;
                if (!ticketValue) {
                    return;
                }
                var sourceValue = this.sourceDropdown.value;
                var selectedTicket  = undefined;
				for(var i=0; i<this.ticketDropdown.selections.length;i++){
	   				var obj = this.ticketDropdown.selections[i];
					if(obj.value == ticketValue)selectedTicket= obj
				}

                var selectedSource = undefined;
				for(var i=0; i<this.sourceDropdown.selections.length;i++){
	   				var obj = this.sourceDropdown.selections[i];
					if(obj.value == sourceValue)selectedSource= obj
				}

                var languageDirectionsFiltedBySource;
                if (selectedSource != null) {
                    languageDirectionsFiltedBySource = selectedTicket.languageDirections.filter(function(ld){return ld.sourceLanguage == selectedSource.globalLinkLocale});
                }
                else {
                    languageDirectionsFiltedBySource = selectedTicket.languageDirections;
                }

                var avaiableTargets = languageDirectionsFiltedBySource.map(function(obj) {
                    return obj.targetLanguage;
                });

                avaiableTargets = avaiableTargets.filter(function (item, pos) {
                    return avaiableTargets.indexOf(item) == pos;
                });

                var selectedTarget = this.targetsDropdown.value;
                if (selectedTarget != null && selectedTarget.length > 0) {
                    var numberOfSelectedTargets = 0;
                    var arSelectedTarget = selectedTarget.split(',');
                    for (var i = 0; i < this.targetsDropdown.selections.length; i++) {
                        var selectionTarget = this.targetsDropdown.selections[i];
                        if (arSelectedTarget.indexOf(selectionTarget.value) > -1 && avaiableTargets.indexOf(selectionTarget.globalLinkLocale) > -1) {
                            numberOfSelectedTargets++;
                        }
                    }

                    if (numberOfSelectedTargets == 0) {
                        this.targetsDropdown.set("value", null);
                    }
                }

                var self = this;
                this.targetsDropdown.set("filter", function (target) {
                    return avaiableTargets.indexOf(target.globalLinkLocale) != -1;
                });
            },

            _updateAttributesDropdown: function () {
                var ticketValue = this.ticketDropdown.value;
                if (!ticketValue) {
                    return;
                }
                var selectedTicket = undefined;
                for (var i = 0; i < this.ticketDropdown.selections.length; i++) {
                    var obj = this.ticketDropdown.selections[i];
                    if (obj.value == ticketValue) selectedTicket = obj
                }

                var ca = selectedTicket.customAttributes;
                var avaiableTargets = [];
                if (ca && ca.length > 0) {
                    avaiableTargets = ca.map(function (obj) {
                        return obj.name;
                    });
                }

                var self = this;
                this.customAttributes.set("filter", function (target) {
                    return avaiableTargets.indexOf(target.name) != -1;
                });
            }
        });
    });